#version 430 core

in vec3 worldNormal;
in vec3 fragmentPos;
in vec2 fragmentUV;

layout(location = 0) out vec4 outColor;

struct Material {
    vec3 ambient;
    vec3 diffuse;
    vec3 specular;
    float shininess;
}; 
  
struct Light {
    vec3 ambient;
    vec3 diffuse;
    vec3 specular;
    vec3 position;
};

uniform Material material;
uniform Light light;

uniform sampler2D color_texture;
uniform vec3 view_dir;

void main() {

	vec3 norm = normalize(worldNormal);
    vec3 lightDir = normalize(light.position - fragmentPos);

	vec3 viewDir = normalize(-view_dir);
    vec3 reflectDir = reflect(-lightDir, norm);

    vec3 ambient = light.ambient * material.ambient;
  	 
    float diff = max(dot(norm, lightDir), 0.0);
    vec3 diffuse = light.diffuse * (diff * material.diffuse);
	 
    float spec = pow(max(dot(viewDir, reflectDir), 0.0), material.shininess);
    vec3 specular = light.specular * (spec * material.specular);  
    
	vec3 textureColor = texture(color_texture, fragmentUV).rgb;
	outColor = vec4((ambient + diffuse + specular) * textureColor, 1.0);

}