#version 430 core

out vec4 FragColor;

in VS_OUT {
    vec3 FragPos;
    vec3 Normal;
    vec2 TexCoords;
} fs_in;

struct Material {
    vec3 ambient;
    vec3 diffuse;
    vec3 specular;
    float shininess;
}; 
  
struct Light {
    vec3 ambient;
    vec3 diffuse;
    vec3 specular;
    vec3 position;
};

uniform Material material;
uniform Light light;

uniform sampler2D diffuseTexture;
uniform samplerCube depthCube;

uniform vec3 viewPos;
uniform vec3 lightPos;

uniform float far_plane;
uniform bool transparency;

void main() {           
   
    vec3 normal = normalize(fs_in.Normal);
	vec3 lightDir = normalize(light.position - fs_in.FragPos);

    // ambient term
    vec3 ambient = light.ambient * material.ambient;

    // diffuse term
    float diff = max(dot(lightDir, normal), 0.0f);
    vec3 diffuse = light.diffuse * diff * material.diffuse;

    // specular term
    vec3 viewDir = normalize(viewPos - fs_in.FragPos);
    vec3 H = normalize(lightDir + viewDir);
    float spec = pow(max(dot(normal, H), 0.0f), material.shininess);
	vec3 specular = light.specular * (spec * material.specular);

    // shadow calculation
	vec3 fragToLight = fs_in.FragPos - light.position;
	float closestDepth = texture(depthCube, fragToLight).r * far_plane;
	float currentDepth = length(fragToLight);

	vec3 gridSamplingDisk[20] = vec3[] ( //offset directions needed for pcfs
		vec3(1, 1, 1), vec3(1, -1, 1), vec3(-1, -1, 1), vec3(-1, 1, 1), 
		vec3(1, 1, -1), vec3(1, -1, -1), vec3(-1, -1, -1), vec3(-1, 1, -1),
		vec3(1, 1, 0), vec3(1, -1, 0), vec3(-1, -1, 0), vec3(-1, 1, 0),
		vec3(1, 0, 1), vec3(-1, 0, 1), vec3(1, 0, -1), vec3(-1, 0, -1),
		vec3(0, 1, 1), vec3(0, -1, 1), vec3(0, -1, -1), vec3(0, 1, -1)
	);

	float shadow = 0.0f;
    float bias = 0.2f;
    int samples = 20;
    float viewDistance = length(viewPos - fs_in.FragPos);
    float diskRadius = (1.0 + (viewDistance / far_plane)) / 25.0f;

    for(int i = 0; i < samples; ++i) {
        float closestDepth = texture(depthCube, fragToLight + gridSamplingDisk[i] * diskRadius).r * far_plane;
        if(currentDepth - bias > closestDepth) shadow += 0.8f;
    }
    shadow /= float(samples);

	if(length(fragToLight) > far_plane) shadow = 0.0f;

	if(!transparency) {
		vec3 texColor = texture(diffuseTexture, fs_in.TexCoords).rgb;
		vec3 lighting = (ambient + (1.0f - shadow) * (diffuse + specular)) * texColor;
		FragColor = vec4(lighting, 1.0f);

	} else {
		vec3 lighting = (ambient + (1.0f - shadow) * (diffuse + specular));    
		FragColor = texture(diffuseTexture, fs_in.TexCoords) * vec4(lighting, 1.0f);
	
	}

	//FragColor = vec4(vec3(closestDepth / far_plane), 1.0);


}